#define  USE_HY17M26_2M
/******************************************************************************
 * BIE_Demo.c                                                        *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *   BIE Address                                                              *
 * addr[0x00]=0x0123                                                          *
 * addr[0x01]=0x4567                                                          *
 * addr[0x02]=0x89AB                                                          *
 * addr[0x03]=0xCDEF                                                          *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : BIE                                                         *
 * Created Date : 2021/6/4 BY Cyril                                           *
 *                                                                            *
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <BIE.h>
#include <CLK.h>
#include <RST.h>


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned int BIE_ReadData[10];
unsigned char Addr;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{   unsigned char i;

    BIE1_Enable();
    BIE1_Erase();      //Erase BIE(addr 0x00~0x7F), total 128 Word
    for(i=0;i<=9;i++)
    {
        BIE_ReadData[i]=BIE1_ReadData(i);  // Read Data
    }

    //BIE Write
    Addr=0x00;
    BIE1_WriterData(Addr,0x01,0x23);  //Addr=0x00, DataH=0x01, DataL=0x23
    BIE1_WriterData(Addr+1,0x45,0x67);  //Addr=0x01, DataH=0x45, DataL=0x67
    BIE1_WriterData(Addr+2,0x89,0xAB);  //Addr=0x02, DataH=0x89, DataL=0xAB
    BIE1_WriterData(Addr+3,0xCD,0xEF);  //Addr=0x03, DataH=0xCD, DataL=0xEF

    //BIE Read
    for(i=0;i<=9;i++)
    {
        BIE_ReadData[i]=BIE1_ReadData(i);  // Read Data
    }

    while(1);
}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* Subroutines Function                                                       */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
