#define  USE_HY17M26_2M
/******************************************************************************
 * I2C_Master_Demo.c                                                          *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        HY17M26               24C02A                                        *
 *---------------------        ---------------                                *
 *                    |       |                                               *
 *                    |       |                                               *
 *                VDD |-----> | VDD                                           *
 *              PT3.7 |-----> | SCL                                           *
 *              PT3.6 |<----> | SDA                                           *
 *                VSS |-----> | VSS                                           *
 *                    |       |                                               *
 *                    |       |                                               *
 *--------------------         ---------------                                *
 *                                                                            *
 * EEPROM Address                                                             *
 * addr[0x00]=0xaa                                                            *
 * addr[0x01]=0x02                                                            *
 * addr[0x02]=0x03                                                            *
 * addr[0x03]=0x04                                                            *
 * addr[0x04]=0x05                                                            *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : I2C_Master                                                  *
 * Created Date : 2021/6/4 BY Cyril                                           *
 *                                                                            *
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <CLK.h>
#include <RST.h>
#include <I2C.h>
#include <GPIO.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif

//I2C PORT
//#define SCL_PT31
//#define SCL_PT33
//#define SCL_PT35
#define SCL_PT37

#define EEPROM_ADDR    0xA0  // Device address for slave target

#define I2CBufferSize  10
#define I2C_WRITE  0x00       // I2C WRITE command
#define I2C_READ   0x01       // I2C READ command
#define I2C_Delay   100000  //@2MHz, Delay 5s
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void I2CInit(void);
void CLOCKInit(void);
void EEPROM_ByteWrite(unsigned int addr, unsigned char dat);
unsigned char EEPROM_ByteRead(unsigned int addr);
void EEPROM_WriteArray(unsigned int dest_addr, unsigned char* src_addr,
                       unsigned int len);
void EEPROM_WriteArray1(unsigned int dest_addr, unsigned char* src_addr,
                       unsigned int len);
void EEPROM_ReadArray(unsigned char* dest_addr, unsigned int src_addr,
                      unsigned int len);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char I2C_Read_Buffer[I2CBufferSize];
unsigned char I2C_RW;
unsigned char I2C_TARGET;      // Target I2C slave address
unsigned char I2C_Sendbuf[I2CBufferSize];
unsigned char I2C_Recbuf[I2CBufferSize];
unsigned char I2C_EndFlag;
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;
unsigned long I2C_Timeout;
unsigned char EEPROM_WriteData[4] = {0x02,0x03,0x04,0x05};
unsigned char i,temp;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
    CLOCKInit();
    I2CInit();

	for(i=0;i<I2CBufferSize;i++)
	{
		I2C_Read_Buffer[i]=0x00;	//CLR I2C_Read_Buffer[]
	}

	GIE_Enable();

	// I2C single I2C_WRITE & I2C_READ
	EEPROM_ByteWrite(0x00,0xAA);               //Single Byte I2C_WRITE word address 0x00, and I2C_WRITE data 0x01
	Delayms(10);                               //Delay for EEPROM 24C02 I2C_WRITE Cycle Time
	I2C_Read_Buffer[0]=EEPROM_ByteRead(0x00);  //Single Byte I2C_READ word address 0x00, the I2C_READ value is 0x01

	// I2C sequential I2C_WRITE & I2C_READ
	EEPROM_WriteArray(0x01,EEPROM_WriteData,4); //I2C_WRITE array data to the word address from 0x01 to 0x04
	Delayms(10);                                //Delay for EEPROM 24C02 I2C_WRITE Cycle Time
	EEPROM_ReadArray(I2C_Read_Buffer, 0x00, 2); //sequential I2C_READ data from 0x00 to 0x01
	EEPROM_ReadArray(I2C_Read_Buffer, 0x00, 5); //sequential I2C_READ data from 0x00 to 0x04
	EEPROM_ReadArray(I2C_Read_Buffer, 0x00, 6); //sequential I2C_READ data from 0x00 to 0x05

	while(1);
}
/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
	if(I2CIF_IsFlag() &&I2C_EndFlag==0)  //Get I2C Interrupt Flag
	{
    	switch(I2C0_STAState())
    	{
	      case 0x90:  //MACTFlag+RWFlag
	                 //START has been transmitted
	            I2C0_SendData(I2C_TARGET);  //Send Slave Address & R/W Bit
	            I2C0_Ctrl(0,0,0,0);          //Clear all I2C flag
	            break;

	      case 0x84:  //MACTFlag+ACKFlag
	                 //Slave A + W has been transmitted. ACK has been received.
	            I2C0_SendData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
	            I2C0_Ctrl(0,0,0,0);      //Clear all I2C flag
	            break;

	      case 0x80: //MACTFlag
	                //Slave A + W has been transmitted. ACK has been received.
	      		I2C0_SendData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
	            I2C0_Ctrl(0,0,0,0);      //Clear all I2C flag
	            break;

	      case 0x8C:  //MACTFlag+DFFlag+ACKFlag
	                //DATA has been transmitted and ACK has been received
	            if(I2C_DataTxIndex<I2C_DataTxLen)
	            {
	              I2C0_SendData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
	              I2C0_Ctrl(0,0,0,0);    // Clear all I2C flag
	            }
	            else
	            {
		          if(I2C_RW == I2C_WRITE)
		          {
		            I2C0_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
		            I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
		            while(I2C_Timeout != 0)
		            {
			          I2C_Timeout--;
		              if(STA0== 0x30)     //A STOP has been transmitted
		              {
		                I2C_EndFlag=1;
		                break;
			          }
		            }
		          }
		          else if(I2C_RW == I2C_READ)
		            I2C0_Ctrl(1,0,0,0);  //I2C as master sends START signal
		            I2C_DataTxIndex=0;
	            }
	            break;

	      case 0x88:  //MACTFlag+DFFlag
	                 //DATA has been transmitted and NACK has been received
	            I2C0_Ctrl(0,1,0,0);    //I2C as master sends STOP signal
	            I2C_DataTxIndex=0;
                I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
                while(I2C_Timeout != 0)
                {
	              I2C_Timeout--;
                  if(STA0== 0x30)     //A STOP has been transmitted
                  {
                    I2C_EndFlag=1;
                    break;
	              }
                }
	            break;

	      case 0xB0:
	                 //A repeated START has been transmitted.
	            I2C0_SendData(I2C_TARGET | I2C_READ);  //Send Slave Address & R/W Bit
	            I2C0_Ctrl(0,0,0,0);    //Clear all I2C flag
	            break;

	      case 0x94:  //MACTFlag+RWFlag+ACKFlag
	                 //Slave A + R has been transmitted. ACK has been received.
	            if(I2C_DataRxLen>1)
	            {
	              I2C0_Ctrl(0,0,0,1);  //Set ACK bit
	            }
                else
                {
	              I2C0_Ctrl(0,0,0,0);  //Clear all I2C flag
	            }
	            break;

	      case 0x9C:  //MACTFlag+RWFlag+DFFlag+ACKFlag
	                //Data byte has been received. ACK has been transmitted.
	            I2C0_ReceiveDATA(I2C_Recbuf[I2C_DataRxIndex++]);
	            if((I2C_DataRxLen-1)>I2C_DataRxIndex)
	            {
	              I2C0_Ctrl(0,0,0,1);  //Set ACK bit
	            }
	            else
	            {
	              I2C0_Ctrl(0,0,0,0);  //Clear all I2C flag
	            }
	            break;

	      case 0x98:  //MACTFlag+RWFlag+DFFlag
	                //Data byte has been received. NACK has been transmitted.
	            I2C0_ReceiveDATA(I2C_Recbuf[I2C_DataRxIndex++]);
	            I2C0_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
                I2C_Timeout=I2C_Delay;  //HAO=2MHz, wait 5s
                while(I2C_Timeout != 0)
                {
	              I2C_Timeout--;
                  if(STA0== 0x30)     //A STOP has been transmitted
                  {
                    I2C_EndFlag=1;
                    break;
	              }
                }
                break;

	      default:
	            I2C0_Ctrl(0,0,0,0);  //Clear all I2C flag
	            I2C_EndFlag=1;
	            break;
    	}
		I2C0_I2CINT_CLEAR();		//CLR I2C INT Flag
		I2C0_I2CER_CLEAR();		//CLR I2C error Flag
		I2CIF_ClearFlag();				//CLR I2CIF
  	}

  	if(I2CERIF_IsFlag())  //Get I2C Error Interrupt Flag
  	{
	    I2C_EndFlag=1;
		I2C0_I2CINT_CLEAR();		//CLR I2C INT Flag
		I2C0_I2CER_CLEAR();		//CLR I2C error Flag
		I2CERIF_ClearFlag();		//CLR I2CERIF
	    I2C0_Ctrl(0,0,0,0);          //Clear all I2C flag
  	}

}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* I2C Init Function                                                        */
/*----------------------------------------------------------------------------*/
void I2CInit(void)
{
  //GPIO Init
#if defined(SCL_PT31)
  GPIO_GSCLSet(GSCL_PT31);
#endif
#if defined(SCL_PT33)
  GPIO_GSCLSet(GSCL_PT33);
#endif
#if defined(SCL_PT35)
  GPIO_GSCLSet(GSCL_PT35);
#endif
#if defined(SCL_PT37)
  GPIO_GSCLSet(GSCL_PT37);
#endif

#if defined(HAO_2MHZ)
  I2C0_Open(40);  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(4+1)]=100kHz
#endif
#if defined(HAO_4MHZ)
  I2C0_Open(9);  //Default CPU clock is 1MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 4000000/[4*(9+1)]=100kHz
#endif
#if defined(HAO_8MHZ)
  I2C0_Open(19);  //Default CPU clock is 8MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 8000000/[4*(19+1)]=100kHz
#endif
#if defined(HAO_16MHZ)
  I2C0_Open(39);  //Default CPU clock is 16MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 16000000/[4*(39+1)]=100kHz
#endif

}

/*----------------------------------------------------------------------------*/
/* EEPROM_ByteWrite ()                                                        */
/*                                                                            */
/* Return Value : None                                                        */
/* Parameters   :                                                             */
/* 1) unsigned int addr - address to I2C_WRITE in the EEPROM                  */
/*                                                                            */
/* 2) unsigned char dat-data to I2C_WRITE to the address <addr> in the EEPROM */
/*                        range is full range of character: 0 to 255          */
/*                                                                            */
/* This function writes the value in <dat> to location <addr> in the EEPROM   */
/* then polls the EEPROM until the I2C_WRITE is complete.                     */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void EEPROM_ByteWrite(unsigned int addr, unsigned char dat)
{
  I2C_TARGET  = EEPROM_ADDR;   //Device_ID_Addr=0XA0

  I2C_Sendbuf[0]  = (addr&0xFF);  //24Cxx Memory Address
  I2C_Sendbuf[1]  = dat;          //24Cxx Memory Data
  I2C_DataTxLen=2;
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  I2C_RW = I2C_WRITE;

  I2C0_Ctrl(1,0,0,0);  // I2C as master sends START signal
  while(I2C_EndFlag==0);

}

/*----------------------------------------------------------------------------*/
/* EEPROM_ByteRead ()                                                         */
/*----------------------------------------------------------------------------*/
unsigned char EEPROM_ByteRead(unsigned int addr)
{
  unsigned char retval;               // Holds the return value

  I2C_TARGET  = EEPROM_ADDR;  //Device_ID_Addr=0XA0

  I2C_Sendbuf[0]  = (addr&0xFF);  //24Cxx Memory Address
  I2C_DataTxLen=1;
  I2C_DataTxIndex=0;
  I2C_DataRxLen=1;
  I2C_DataRxIndex=0;
  I2C_EndFlag=0;
  I2C_RW = I2C_READ;

  I2C0_Ctrl(1,0,0,0);  // I2C as master sends START signal
  while(I2C_EndFlag==0);

  retval=I2C_Recbuf[0];
  return retval;
}

/*----------------------------------------------------------------------------*/
/* EEPROM_WriteArray()                                                       */
/*----------------------------------------------------------------------------*/
/* Return Value : None                                                        */
/* Parameters   :                                                             */
/* 1) unsigned int dest_addr-beginning address to I2C_WRITE to in the EEPROM  */
/* 2) unsigned char* src_addr - pointer to the array of data to be written    */
/*                        range is full range of character: 0 to 255          */
/* 3) unsigned int len - length of the array to be written to the EEPROM      */
/* Writes <len> data bytes to the EEPROM slave specified by the <EEPROM_ADDR> */
/* constant.                                                                  */
/*----------------------------------------------------------------------------*/
void EEPROM_WriteArray(unsigned int dest_addr, unsigned char* src_addr, unsigned int len)
{
  unsigned char* pData = (unsigned char*) src_addr;
  unsigned int WCount;

  I2C_TARGET  = EEPROM_ADDR;
  I2C_Sendbuf[0]  = (dest_addr&0xFF);
  I2C_DataTxLen=1;

  for(WCount=0 ; WCount<len; WCount++)
  {
    I2C_Sendbuf[I2C_DataTxLen++]=*pData;
    pData++;
  }
  I2C_DataTxIndex=0;
  I2C_EndFlag=0;
  I2C_RW = I2C_WRITE;

  I2C0_Ctrl(1,0,0,0);  // I2C as master sends START signal
  while(I2C_EndFlag==0);

}

/*----------------------------------------------------------------------------*/
/* EEPROM_ReadArray ()                                                        */
/*----------------------------------------------------------------------------*/
/* Return Value : None                                                        */
/* Parameters   :                                                             */
/*   1) unsigned char* dest_addr - pointer to the array that will be filled   */
/*                                 with the data from the EEPROM              */
/*                        range is full range of character: 0 to 255          */
/* 2)unsigned int src_addr-beginning address to I2C_READ data from the EEPROM */
/* 3)unsigned int len - length of the array to be I2C_READ from the EEPROM    */
/* Reads up to 256 data bytes from the EEPROM slave specified by the          */
/* <EEPROM_ADDR> constant.                                                    */
/*----------------------------------------------------------------------------*/
void EEPROM_ReadArray (unsigned char* dest_addr, unsigned int src_addr,
                       unsigned int len)
{
  unsigned char* pData = (unsigned char*) dest_addr;
  unsigned int RCount;

  I2C_TARGET  = EEPROM_ADDR;  //Device_ID_Addr=0XA0

  I2C_Sendbuf[0]  = (src_addr&0xFF);  //24Cxx Memory Address
  I2C_DataTxLen=1;
  I2C_DataTxIndex=0;
  I2C_DataRxLen=len;
  I2C_DataRxIndex=0;
  I2C_EndFlag=0;
  I2C_RW = I2C_READ;

  I2C0_Ctrl(1,0,0,0);  // I2C as master sends START signal
  while(I2C_EndFlag==0);

  for(RCount=0 ; RCount < len ; RCount++ )
  {
    *pData=I2C_Recbuf[RCount];
    pData++;
  }
}
/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
