#define  USE_HY17M26_2M
/******************************************************************************
 * I2C_Slave_Demo.c                                                           *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        Host                      HY17M26                                   *
 *---------------------        ------------------                             *
 *                     |      |                                               *
 *                     |      |                                               *
 *                     | INP<-| AI0                                           *
 *                     | INN<-| AI1                                           *
 *                     |      |                                               *
 *                 VDD |----> | VDD                                           *
 *                 SCL |----> | PT3.7                                         *
 *                 SDA |<---> | PT3.6                                         *
 *                 VSS |----> | VSS                                           *
 *                     |      |                                               *
 *                     |      |                                               *
 *-------------------         ----------------                                *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : I2C_Slave                                                   *
 * Created Date : 2021/6/4 BY Cyril                                           *
 *                                                                            *
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <hy17m26sfr.h>
#include <INT.h>
#include <I2C.h>
#include <CLK.h>
#include <GPIO.h>
#include <PWR.h>
#include <ADC.h>
#include <RST.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif


//I2C PORT
//#define SCL_PT31
//#define SCL_PT33
//#define SCL_PT35
#define SCL_PT37

#define I2CBufferSize  32
#define I2C_Slave_Command 0x80
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
volatile typedef union _Flag
{
 unsigned int  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_I2C_TxDone:1;
    unsigned b_I2C_RxDone:1;
    unsigned Reserved:15;
  };
} Flag;

//signed long
//range 8388607~-8388608 (dec)
//range 0x007F FFFF ~ 0xFF80 0000 (hex)
signed long  temp;
signed long  ADC_ISRTemp;
Flag   Flagbits;

unsigned char I2C_Sendbuf[I2CBufferSize];
unsigned char I2C_Recbuf[I2CBufferSize];
unsigned char I2C_EndFlag;
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;
char ADC_LowByte,ADC_MiddleByte,ADC_HighByte;
unsigned char ADC_Finish;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void ADCInit(void);
void CLOCKInit(void);
void I2CInit(void);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
    CLOCKInit();
    ADCInit();
    I2CInit();

	GIE_Enable();

	while(1)
	{
        if(I2C_EndFlag ==1)
        {
          I2C_EndFlag= 0;
          ADC_Enable();
          ADC_CMFREnable();
        }
        if(Flagbits.b_ADCdone==1)
        {
          Flagbits.b_ADCdone=0;
          I2CIE_Disable();
          I2CERIE_Disable();
          ADC_Disable(); //ENAD1=0b

          //ADC=23bit, +/-22bit, ADC Range : 4194303~-4194304
          ADC_LowByte=(ADC_ISRTemp >> 0) & 0xFF;
          ADC_MiddleByte=(ADC_ISRTemp >> 8) & 0xFF;
          ADC_HighByte=(ADC_ISRTemp >> 16) & 0xFF;
          I2C_Sendbuf[0]=ADC_LowByte;
          I2C_Sendbuf[1]=ADC_MiddleByte;
          I2C_Sendbuf[2]=ADC_HighByte;

          ADC_Finish=1;      //Enable I2C Slave data Buffer Update
          ADC_Enable();//ENAD1=1b
          ADC_CMFREnable(); //CMFR=1b
          ADC_INT_Enable();  //ADIE=1b

          I2CIE_Enable();
          I2CERIE_Enable();
        }
    }
}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
	unsigned char STAData;

    //ADC Event
    if(ADC_INT_IsFlag())
    {
      ADC_INT_ClearFlag();
      ADC_ISRTemp=ADCR; //ADC_GetData();
      Flagbits.b_ADCdone=1;
      ADC_INT_Disable();
    }

    //I2C Event
	if(I2CIF_IsFlag())
	{
		ADC_Disable();      //ADC Disable
		I2CIF_ClearFlag();
		STAData=I2C0_STAState();
		switch(STAData)
		{

			case 0x44:
			{
				I2C_DataRxIndex=0;
				I2C0_Ctrl(0,0,0,1);
				break;
			}
			case 0x48:
			{
				I2C_Recbuf[I2C_DataRxIndex]=RDB0;
				I2C_DataRxIndex++;
				I2C0_Ctrl(0,0,0,0);
				break;
			}
			case 0x4C:
			{
                temp=RDB0;  //IF I2C command=0x80
                I2C_Recbuf[I2C_DataRxIndex] = temp;
                if(I2C_Recbuf[I2C_DataRxIndex]==I2C_Slave_Command)
                {
                    if(ADC_Finish==1)
                    {
                        I2C_Sendbuf[0]|=0x1;      //LowByte bit0=1b, new data
                        ADC_Finish=0;
                    }
                    else
                    {
                        I2C_Sendbuf[0] &=0xFE;    //LowByte bit0=0b, old data
                    }
                }
                I2C_DataRxIndex++;
                I2C0_Ctrl(0,0,0,1);
				break;
			}
			case 0x54:
			{
				I2C_DataTxIndex=0;
				I2C0_SendData(I2C_Sendbuf[I2C_DataTxIndex]);
				I2C_DataTxIndex++;
				I2C0_Ctrl(0,0,0,0);
				break;
			}
			case 0x55:
			{
				I2C_DataTxIndex=0;
				I2C0_SendData(I2C_Sendbuf[I2C_DataTxIndex]);
				I2C_DataTxIndex++;
				I2C0_Ctrl(0,0,0,0);
				break;
			}
			case 0x58:
			{
				I2C_DataTxLen=I2C_DataTxIndex;
                I2C_EndFlag= 1;
                I2C0_SendData(0xff);
				I2C0_Ctrl(0,0,0,0);
				break;
			}
			case 0x5C:
			{
				I2C0_SendData(I2C_Sendbuf[I2C_DataTxIndex]);
				I2C_DataTxIndex++;
				I2C0_Ctrl(0,0,0,0);
				break;
			}
			case 0x30:
			{
				I2C_DataRxLen=I2C_DataRxIndex;
                I2C_DataTxLen=I2C_DataTxIndex;
                I2C_EndFlag= 1;
                I2C0_SendData(0xff);
				I2C0_Ctrl(0,0,0,0);
				break;
			}
            default:
            {
                I2C_DataTxIndex=0;
                I2C_DataRxIndex=0;
                I2C_EndFlag=1;
				I2C0_Ctrl(0,0,0,0);
                break;
            }
		}
	}

}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* ADC Init Function                                                          */
/*----------------------------------------------------------------------------*/
void ADCInit(void)
{
   //==========Setup Power=====================================
  PWR_BGREnable();			//bandgap Vref Enable
  PWR_VDDAOpen(LDOC_2V4);	//VDDA Enable

  //==========Setup ADC Clock=================================
#if defined(HAO_2MHZ)
  //HAO=2MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV2,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_4MHZ)
  //HAO=4MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV4,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_8MHZ)
  //HAO=8MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV8,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_16MHZ)
  //HAO=17MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV16,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif


#if defined ADC_ChopperMode
  CSFON_Enable();
  ADC_ENINXCH_Enable();
  ADC_DAFM_CHOPPER();
  ADC_ENCH_Enable();
  ADC_Enable();
  ADC_CMFREnable();     //CMFR=1, Comb Filter Reset
#endif

}

/*----------------------------------------------------------------------------*/
/* I2C Init Function                                                        */
/*----------------------------------------------------------------------------*/
void I2CInit(void)
{
  //GPIO Init
#if defined(SCL_PT31)
  GPIO_GSCLSet(GSCL_PT31);
#endif
#if defined(SCL_PT33)
  GPIO_GSCLSet(GSCL_PT33);
#endif
#if defined(SCL_PT35)
  GPIO_GSCLSet(GSCL_PT35);
#endif
#if defined(SCL_PT37)
  GPIO_GSCLSet(GSCL_PT37);
#endif

#if defined(HAO_2MHZ)
  I2C0_Open(40);  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(4+1)]=100kHz
#endif
#if defined(HAO_4MHZ)
  I2C0_Open(9);  //Default CPU clock is 1MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 4000000/[4*(9+1)]=100kHz
#endif
#if defined(HAO_8MHZ)
  I2C0_Open(19);  //Default CPU clock is 8MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 8000000/[4*(19+1)]=100kHz
#endif
#if defined(HAO_16MHZ)
  I2C0_Open(39);  //Default CPU clock is 16MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 16000000/[4*(39+1)]=100kHz
#endif

  I2C0_SlaveSet(0xA0);        //I2C Slave address=0xA0
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
