#define  USE_HY17M26_2M
/******************************************************************************
 * MFC_Demo.c                                                                 *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        HY17M26                                                             *
 *--------------------                                                        *
 *                    |                                                       *
 *               PT3.6|->Output                                               *
 *                    |                                                       *
 *               PT3.7|<-Input                                                *
 *                    |                                                       *
 *--------------------                                                        *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : MFC                                                         *
 * Created Date : 2021/6/4 BY Cyril                                           *
 *                                                                            *
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <PWR.h>
#include <GPIO.h>
#include <RST.h>
#include <ADC.h>
#include <CLK.h>
#include <CMP.h>

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void CMP_LVDInit(void);
void CMP_HysteresisInit(void);
void CLOCKInit(void);
void GPIOInit(void);
void Delay(unsigned int num);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
    CMP_LVDInit();
    //CMP_HysteresisInit();

	GIE_Enable();
	while(1);
}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{

}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* LVD Mode Init Function                                                      */
/*----------------------------------------------------------------------------*/
void CMP_LVDInit(void)
{
  CMP_RLOSet(CPRH_VDD,CPRL_OPEN,CPDA_26DIV32,CPDM_DISABLE );  //LVD=1.92V (When VDD >1.92V PT36=Low, When VDD<1.92V PT36=High)
  //Set CPDA[4:0]
  //CPDA_6DIV32, LVD=3.39V
  //CPDA_7DIV32, LVD=3.27V
  //CPDA_8DIV32, LVD=3.15V
  //CPDA_9DIV32, LVD=3.04V
  //CPDA_10DIV32, LVD=2.94V
  //CPDA_11DIV32, LVD=2.85V
  //CPDA_12DIV32, LVD=2.76V
  //CPDA_13DIV32, LVD=2.67V
  //CPDA_14DIV32, LVD=2.59V
  //CPDA_15DIV32, LVD=2.52V
  //CPDA_16DIV32, LVD=2.45V
  //CPDA_17DIV32, LVD=2.38V
  //CPDA_18DIV32, LVD=2.32V
  //CPDA_19DIV32, LVD=2.26V
  //CPDA_20DIV32, LVD=2.21V
  //CPDA_21DIV32, LVD=2.15V
  //CPDA_22DIV32, LVD=2.10V
  //CPDA_23DIV32, LVD=2.05V
  //CPDA_24DIV32, LVD=2.01V
  //CPDA_25DIV32, LVD=1.96V
  //CPDA_26DIV32, LVD=1.92V

  CMP_Open(CPPS_1V2,CPNS_RLO,CMPHS_NORMAL,CPOR_NORMAL,ENRCC_PT36);   //RLO= VDD*((26+20)/(22.5+32+20))

}

/*----------------------------------------------------------------------------*/
/* HysteresisInit Mode Init Function                                          */
/*----------------------------------------------------------------------------*/
void CMP_HysteresisInit(void)
{
	PWR_VDDAOpen(LDOC_2V4); // refer to the Datasheet for LDOC typical voltage
	CMP_RLOSet(CPRH_VDDA,CPRL_SHORT,CPDA_24DIV32,0x08);
    //CPDM[4:0]=01000
    //CPDA[4:0]=11000, RLO= VDDA*(24/32) <-switch over-> CPDA[4:0]=10000, RLO= VDDA*(16/32)
    CMP_Open(CPPS_CH2,CPNS_RLO,CMPHS_NORMAL,CPOR_NORMAL,ENRCC_PT36);    // When PT3.7<1.2V PT36=Low, When PT3.7 >1.8V PT36=High,

}

/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}


/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
