#define  USE_HY17M26_2M
/******************************************************************************
 * PWM_Demo.c                                                                 *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *   HY17M26                                                                  *
 * -----------                                                                *
 *           |                                                                *
 *     PT3.4 |-->PWM0                                                         *
 *           |                                                                *
 *     PT3.5 |-->PWM1                                                         *
 *           |                                                                *
 *     PT1.1 |-->PWM2                                                         *
 *           |                                                                *
 *     PT1.2 |-->PWM3                                                         *
 *           |                                                                *
 * -----------                                                                *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : PWM                                                         *
 * Created Date : 2021/6/4 By Cyril                                           *
 *                                                                            *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <TMR.h>
#include <CLK.h>
#include <INT.h>
#include <RST.h>
#include <GPIO.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif

////	Select PWM MODE		//
//#define PWM1O
//#define PWM2O
//#define PWM3O
//#define PWM4O
//#define PWM5O
//#define PWM6O
#define PWM7O

//TMB_PWM PORT
//PWM0
//#define PWM0_PT30
//#define PWM0_PT32
#define PWM0_PT34
//#define PWM0_PT36

//PWM1
//#define PWM1_PT31
//#define PWM1_PT33
#define PWM1_PT35
//#define PWM1_PT37

//PWM2
#define PWM2_PT11
//#define PWM2_PT13
//#define PWM2_PT20
//#define PWM2_PT22

//PWM3
#define PWM3_PT12
//#define PWM3_PT14
//#define PWM3_PT21
//#define PWM3_PT23
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void CLOCKInit(void);
void PWMInit(void);

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
	CLOCKInit();
	PWMInit();


	while(1);

}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* PWM Init Function                                                        */
/*----------------------------------------------------------------------------*/
void PWMInit(void)
{
#if defined(PWM0_PT30)
  GPIO_GPWM0Set(GPWM0_PT30);
#endif
#if defined(PWM0_PT32)
  GPIO_GPWM0Set(GPWM0_PT32);
#endif
#if defined(PWM0_PT34)
  GPIO_GPWM0Set(GPWM0_PT34);
#endif
#if defined(PWM0_PT36)
  GPIO_GPWM0Set(GPWM0_PT36);
#endif

#if defined(PWM1_PT31)
  GPIO_GPWM1Set(GPWM1_PT31);
#endif
#if defined(PWM1_PT33)
  GPIO_GPWM1Set(GPWM1_PT33);
#endif
#if defined(PWM1_PT35)
  GPIO_GPWM1Set(GPWM1_PT35);
#endif
#if defined(PWM1_PT37)
  GPIO_GPWM1Set(GPWM1_PT37);
#endif

#if defined(PWM2_PT11)
  GPIO_GPWM2Set(GPWM2_PT11);
#endif
#if defined(PWM2_PT13)
  GPIO_GPWM2Set(GPWM2_PT13);
#endif
#if defined(PWM2_PT20)
  GPIO_GPWM2Set(GPWM2_PT20);
#endif
#if defined(PWM2_PT22)
  GPIO_GPWM2Set(GPWM2_PT22);
#endif

#if defined(PWM3_PT12)
  GPIO_GPWM3Set(GPWM3_PT12);
#endif
#if defined(PWM3_PT14)
  GPIO_GPWM3Set(GPWM3_PT14);
#endif
#if defined(PWM3_PT21)
  GPIO_GPWM3Set(GPWM3_PT21);
#endif
#if defined(PWM3_PT23)
  GPIO_GPWM3Set(GPWM3_PT23);
#endif

//***** setting the timer B & PWM *********//
/*--------------------------------------PWM1O-------------------------------------------*/
#ifdef PWM1O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_16bit ,TB1RT_LogicH );
	TMB2_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB2M_16bit ,TB2RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal
	TB2_PWM2_PHASE(PA2IV_NORMAL);		//PWM2 Output Phase Normal
	TB2_PWM3_PHASE(PA3IV_INVER);		//PWM3 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM1O);		//PWM0 Output mode Select PWM1O
	TB1_PWM1ModeSelect(PWMA1_PWM1O);		//PWM1 Output mode Select PWM1O
	TB2_PWM2ModeSelect(PWMA2_PWM1O);		//PWM2 Output mode Select PWM1O
	TB2_PWM3ModeSelect(PWMA3_PWM1O);		//PWM3 Output mode Select PWM1O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable
	TB2_PWMO2(PWMO2_OUTPUT);				//PWM2 Enable
	TB2_PWMO3(PWMO3_OUTPUT);				//PWM3 Enable

	TB1C0Set(0x0800);	// TB1C0[15:0] is setting the PWM frequency.
	TB1C1Set(0x0400);	// TB1C1[15:0] is setting the PWM Duty Cycle.
	TB2C0Set(0x0800);	// TB2C0[15:0] is setting the PWM frequency.
	TB2C1Set(0x0400);	// TB2C1[15:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM2O-------------------------------------------*/
#ifdef PWM2O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_16bit ,TB1RT_LogicH );
	TMB2_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB2M_16bit ,TB2RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal
	TB2_PWM2_PHASE(PA2IV_NORMAL);		//PWM2 Output Phase Normal
	TB2_PWM3_PHASE(PA3IV_INVER);		//PWM3 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM2O);		//PWM0 Output mode Select PWM2O
	TB1_PWM1ModeSelect(PWMA1_PWM2O);		//PWM1 Output mode Select PWM2O
	TB2_PWM2ModeSelect(PWMA2_PWM2O);		//PWM2 Output mode Select PWM2O
	TB2_PWM3ModeSelect(PWMA3_PWM2O);		//PWM3 Output mode Select PWM2O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable
	TB2_PWMO2(PWMO2_OUTPUT);				//PWM2 Enable
	TB2_PWMO3(PWMO3_OUTPUT);				//PWM3 Enable

	TB1C0Set(0x0833);	// TB1C0[15:0] is setting the PWM frequency.
	TB1C2Set(0x01a4);	// TB1C2[15:0] is setting the PWM Duty Cycle.
	TB2C0Set(0x0833);	// TB2C0[15:0] is setting the PWM frequency.
	TB2C2Set(0x01a4);	// TB2C2[15:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM3O-------------------------------------------*/
#ifdef PWM3O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_2_8bit ,TB1RT_LogicH );
	TMB2_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB2M_2_8bit ,TB2RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal
	TB2_PWM2_PHASE(PA2IV_NORMAL);		//PWM2 Output Phase Normal
	TB2_PWM3_PHASE(PA3IV_INVER);		//PWM3 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM3O);		//PWM0 Output mode Select PWM3O
	TB1_PWM1ModeSelect(PWMA1_PWM3O);		//PWM1 Output mode Select PWM3O
	TB2_PWM2ModeSelect(PWMA2_PWM3O);		//PWM2 Output mode Select PWM3O
	TB2_PWM3ModeSelect(PWMA3_PWM3O);		//PWM3 Output mode Select PWM3O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable
	TB2_PWMO2(PWMO2_OUTPUT);				//PWM2 Enable
	TB2_PWMO3(PWMO3_OUTPUT);				//PWM3 Enable

	TB1C0Set(0x0029);	// TB1C0L[7:0] is setting the PWM frequency.
	TB1C1Set(0x0016);	// TB1C0L[7:0] is setting the PWM Duty Cycle.
	TB2C0Set(0x0029);	// TB2C0L[7:0] is setting the PWM frequency.
	TB2C1Set(0x0016);	// TB2C0L[7:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM4O-------------------------------------------*/
#ifdef PWM4O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_2_8bit ,TB1RT_LogicH );
	TMB2_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB2M_2_8bit ,TB2RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal
	TB2_PWM2_PHASE(PA2IV_NORMAL);		//PWM2 Output Phase Normal
	TB2_PWM3_PHASE(PA3IV_INVER);		//PWM3 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM4O);		//PWM0 Output mode Select PWM4O
	TB1_PWM1ModeSelect(PWMA1_PWM4O);		//PWM1 Output mode Select PWM4O
	TB2_PWM2ModeSelect(PWMA2_PWM4O);		//PWM2 Output mode Select PWM4O
	TB2_PWM3ModeSelect(PWMA3_PWM4O);		//PWM3 Output mode Select PWM4O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable
	TB2_PWMO2(PWMO2_OUTPUT);				//PWM2 Enable
	TB2_PWMO3(PWMO3_OUTPUT);				//PWM3 Enable

	TB1C0Set(0xD100);	// TB1C0H[15:8] is setting the PWM frequency.
	TB1C2Set(0x0088);	// TB1C2L[7:0] is setting the PWM Duty Cycle.
	TB2C0Set(0xD100);	// TB2C0H[15:8] is setting the PWM frequency.
	TB2C2Set(0x0088);	// TB2C2L[7:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM5O-------------------------------------------*/
#ifdef PWM5O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_8_8bit ,TB1RT_LogicH );
	TMB2_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB2M_8_8bit ,TB2RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal
	TB2_PWM2_PHASE(PA2IV_NORMAL);		//PWM2 Output Phase Normal
	TB2_PWM3_PHASE(PA3IV_INVER);		//PWM3 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM5O);		//PWM0 Output mode Select PWM5O
	TB1_PWM1ModeSelect(PWMA1_PWM5O);		//PWM1 Output mode Select PWM5O
	TB2_PWM2ModeSelect(PWMA2_PWM5O);		//PWM2 Output mode Select PWM5O
	TB2_PWM3ModeSelect(PWMA3_PWM5O);		//PWM3 Output mode Select PWM5O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable
	TB2_PWMO2(PWMO2_OUTPUT);				//PWM2 Enable
	TB2_PWMO3(PWMO3_OUTPUT);				//PWM3 Enable

	TB1C0Set(0x00C8);	// TB1C0L[7:0] is setting the PWM frequency.
	TB1C1Set(0x0064);	// TB1C1L[7:0] is setting the PWM Duty Cycle.
	TB1C2Set(0x0080);	// TB1C2L[7:0] is setting the PWM Duty Cycle fine tuning.
	TB2C0Set(0x00C8);	// TB2C0L[7:0] is setting the PWM frequency.
	TB2C1Set(0x0064);	// TB2C1L[7:0] is setting the PWM Duty Cycle.
	TB2C2Set(0x0080);	// TB2C2L[7:0] is setting the PWM Duty Cycle fine tuning.
#endif

/*--------------------------------------PWM6O-------------------------------------------*/
#ifdef PWM6O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_17bit ,TB1RT_LogicH );
	TMB2_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB2M_17bit ,TB2RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal
	TB2_PWM2_PHASE(PA2IV_NORMAL);		//PWM2 Output Phase Normal
	TB2_PWM3_PHASE(PA3IV_INVER);		//PWM3 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM6O);		//PWM0 Output mode Select PWM6O
	TB1_PWM1ModeSelect(PWMA1_PWM6O);		//PWM1 Output mode Select PWM6O
	TB2_PWM2ModeSelect(PWMA2_PWM6O);		//PWM2 Output mode Select PWM6O
	TB2_PWM3ModeSelect(PWMA3_PWM6O);		//PWM3 Output mode Select PWM6O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable
	TB2_PWMO2(PWMO2_OUTPUT);				//PWM2 Enable
	TB2_PWMO3(PWMO3_OUTPUT);				//PWM3 Enable

	TB1C0Set(0x0002);	// TB1C0L[7:0] is setting the PWM frequency.
	TB1C1Set(0x0001);	// TB1C1L[7:0] is setting the PWM Duty Cycle.
	TB1C2Set(0x0080);	// TB1C2L[7:0] is setting the PWM Duty Cycle
	TB2C0Set(0x0002);	// TB2C0L[7:0] is setting the PWM frequency.
	TB2C1Set(0x0001);	// TB2C1L[7:0] is setting the PWM Duty Cycle.
	TB2C2Set(0x0080);	// TB2C2L[7:0] is setting the PWM Duty Cycle
									  // TB1C0 > TB1C2 > TB1C1
#endif

/*--------------------------------------PWM7O-------------------------------------------*/
#ifdef PWM7O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_16bit ,TB1RT_LogicH );
	TMB2_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB2M_16bit ,TB2RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal
	TB2_PWM2_PHASE(PA2IV_NORMAL);		//PWM2 Output Phase Normal
	TB2_PWM3_PHASE(PA3IV_INVER);		//PWM3 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM7O);		//PWM0 Output mode Select PWM7O
	TB1_PWM1ModeSelect(PWMA1_PWM7O);		//PWM1 Output mode Select PWM7O
	TB2_PWM2ModeSelect(PWMA2_PWM7O);		//PWM2 Output mode Select PWM7O
	TB2_PWM3ModeSelect(PWMA3_PWM7O);		//PWM3 Output mode Select PWM7O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable
	TB2_PWMO2(PWMO2_OUTPUT);				//PWM2 Enable
	TB2_PWMO3(PWMO3_OUTPUT);				//PWM3 Enable

	TB1C0Set(0x3FFF);	// TB1C0[15:0] is setting the PWM frequency.
										// PWM Duty Cycle is always 50%
	TB2C0Set(0x3FFF);	// TB2C0[15:0] is setting the PWM frequency.
										// PWM Duty Cycle is always 50%
#endif

	TB1Enable();
	TB2Enable();
}

/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* End Of File                                       */
/*----------------------------------------------------------------------------*/

