#define  USE_HY17M26_2M
/******************************************************************************
 * POWER_Demo.c                                                               *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        HY17M26                                                             *
 *--------------------                                                        *
 *                    |                                                       *
 *               PT1.0|<-button (IDLE)                                        *
 *                    |                                                       *
 *               PT1.1|<-button (SLEEP)                                       *
 *                    |                                                       *
 *--------------------                                                        *
 *                                                                            *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : Power                                                       *
 * Created Date : 2021/6/4 BY Cyril                                           *
 *                                                                            *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <PWR.h>
#include <GPIO.h>
#include <RST.h>
#include <INT.h>
#include <CLK.h>

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void GPIOInit(void);
void CLOCKInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
volatile typedef union _Flag
{
  unsigned int  _byte;
  struct
  {
    unsigned b_PT1INT0done:1;
    unsigned b_PT1INT1done:1;
    unsigned b_PT1INT2done:1;
    unsigned b_PT1INT3done:1;
    unsigned b_PT1INT4done:1;
    unsigned b_PT1INT5done:1;
    unsigned b_PT1INT6done:1;
  };
} Flag;
Flag   Flagbits;
unsigned int counterA;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
  counterA=0;
  CLOCKInit();
  GPIOInit();
  Flagbits.b_PT1INT0done=0;
  Flagbits.b_PT1INT1done=0;

  GIE_Enable();

  while(1)
  {

    if(Flagbits.b_PT1INT0done==1)  //PT1.0
    {
      Flagbits.b_PT1INT0done=0;
      CSFON_Enable();
      ENBOR2_Disable();
      PWR_BGRDisable();
      CLK_IDLE_HAOSet(IDLEM_HAOAuto);  //Disable HAO. When wake up from GPIO Interrupt, auto set OSCS[1:0]=00b=OSC_HAO
      Idle();                          //IDLE mode
      counterA++;
    }

    if(Flagbits.b_PT1INT1done==1)  //PT1.1
    {
      Flagbits.b_PT1INT1done=0;
      CSFON_Enable();
      ENBOR2_Disable();
      PWR_BGRDisable();
      CLK_OSCSelect(OSCS_LPO);
      CLK_HAODisable();
      Sleep();                     //Sleep mode
      counterA--;
    }

  }

}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
  NOP();
  //GPIO Event

  if(E0IF_IsFlag())  //if E0IF=1b
  {
    Flagbits.b_PT1INT0done=1;
    E0IF_ClearFlag();  //PT1.0 E0IF=0b, clear PT1.0 Interrupt Flag
  }

  if(E1IF_IsFlag())  //if E1IF=1b
  {
    Flagbits.b_PT1INT1done=1;
    E1IF_ClearFlag();  //PT1.1 E1IF=0b, clear PT1.1 Interrupt Flag
  }
}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* GPIO Init Function                                                         */
/*----------------------------------------------------------------------------*/
void GPIOInit(void)
{
  //GPIO initial on PT1.0 & PT1.1 (INPUT)
  GPIO_PT1InputMode(PT10);
  GPIO_PT1InputMode(PT11);
  GPIO_PT1DigitalEnable(PT10);
  GPIO_PT1DigitalEnable(PT11);
  GPIO_PT1SETPU(PT10);
  GPIO_PT1SETPU(PT11);
  INTEG0Sel(INTEG0_EDGEFALL);
  INTEG1Sel(INTEG1_EDGEFALL);
  E0IE_Enable();
  E1IE_Enable();
  E0IF_ClearFlag();
  E1IF_ClearFlag();


}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
