#define  USE_HY17M26_2M
/******************************************************************************
 * SPI_Host_Demo.c                                                            *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        HY17M26(host)            slave                                      *
 *---------------------          ----------------                             *
 *                     |        |                                             *
 *                     |        |                                             *
 *                 VDD | -----> | VDD                                         *
 *               PT2.4 | -----> | CS                                          *
 *               PT2.5 | -----> | SCL                                         *
 *               PT2.6 | <----- | MISO                                        *
 *               PT2.7 | -----> | MOSI                                        *
 *                 VSS | -----> | VSS                                         *
 *                     |        |                                             *
 *                     |        |                                             *
 *---------------------          ----------------                             *
 *                                                                            *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : SPI_Host                                                    *
 * Created Date : 2021/6/4 By Cyril                                           *
 *                                                                            *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <GPIO.h>
#include <CLK.h>
#include <SPI.h>
#include <RST.h>
#include <INT.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif

//SPI PORT
//#define SPI_PT11    //PT10 CS, PT11 CK, PT12 MISO, PT13 MOSI
#define SPI_PT25  //PT24 CS, PT25 CK, PT26 MISO, PT27 MOSI
//#define SPI_PT31  //PT30 CS, PT31 CK, PT32 MISO, PT33 MOSI
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void CLOCKInit(void);
void GPIOInit(void);
void SPIInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
volatile typedef union _Flag
{
 unsigned int  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_SPIDone:1;
    unsigned Reserved:14;
  };
} Flag;
Flag   Flagbits;
unsigned int SPI_READ_Buffer[8];
unsigned int SPI_write_buffer[8];
unsigned int SPI_Test_counter;
unsigned char ADC_Start;
signed long  ADCData;
signed long  ADC_LowByte;
signed long ADC_MiddleByte;
signed long ADC_HighByte;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
  //HY17M26 SPI Host sends 0xF0, 0x1F. SPI Slave returns 0x12, 0x34 and set to transmit ADC Data at this time.
  //Next, the SPI Host sends 24 bits (8bit+8bit+8bit) at will.
  //at this time the SPI Slave will return ADC_HighByte, ADC_MiddleByte, ADC_LowByte in sequence

  PT10_HWResetEnable();

  CLOCKInit();
  GPIOInit();
  SPIInit();

  SPI_write_buffer[0]=0xF0;
  SPI_write_buffer[1]=0x1F;
  SPI_write_buffer[2]=0xA0;
  SPI_write_buffer[3]=0xA1;
  SPI_write_buffer[4]=0xA2;
  SPI_Test_counter=0;
  Flagbits.b_SPIDone= 0;
  ADC_Start= 0;
  ADCData=0;
  ADC_LowByte= 0;
  ADC_MiddleByte= 0;
  ADC_HighByte= 0;

  //Delay(100);

  GIE_Enable();


  GPIO_PT2OutputLow(PT24); //PT2.4 output low
  Delay(20);      //have to add, this delay time is wait for HY17M26 SPI slave to prepare the data that needs to be transmitted
  SSPBUF0 = SPI_write_buffer[0];
  while(Flagbits.b_SPIDone==1);
  SPI_READ_Buffer[0]=SSPBUF0;
  GPIO_PT2OutputHigh(PT24); //PT2.4 output high
  Flagbits.b_SPIDone= 0;
  Delay(200); //have to add, otherwise, write to HY17M26 SPI slave fail

  GPIO_PT2OutputLow(PT24); //PT2.4 output low
  Delay(20);      //have to add, this delay time is wait for HY17M26 SPI slave to prepare the data that needs to be transmitted
  SSPBUF0 = SPI_write_buffer[1];
  while(Flagbits.b_SPIDone==1);
  SPI_READ_Buffer[1]=SSPBUF0;
  GPIO_PT2OutputHigh(PT24); //PT2.4 output high
  Flagbits.b_SPIDone= 0;
  Delay(200); //have to add, otherwise, write to HY17M26 SPI slave fail

  while(1)
  {
    if(GPIO_PT2GET(PT20)==0)
    {
      Delay(2000);
      while(GPIO_PT2GET(PT20)==0);

      GPIO_PT2OutputLow(PT24); //PT2.4 output low
      Delay(20);      //have to add, this delay time is wait for HY17M26 SPI slave to prepare the data that needs to be transmitted
      SSPBUF0 = SPI_write_buffer[2];
      while(Flagbits.b_SPIDone==1);
      SPI_READ_Buffer[2]=SSPBUF0;
      ADC_HighByte=SPI_READ_Buffer[2];
      GPIO_PT2OutputHigh(PT24); //PT2.4 output high
      Flagbits.b_SPIDone= 0;
      Delay(200); //have to add, otherwise, write to HY17M26 SPI slave fail

      GPIO_PT2OutputLow(PT24); //PT2.4 output low
      Delay(20);      //have to add, this delay time is wait for HY17M26 SPI slave to prepare the data that needs to be transmitted
      SSPBUF0 = SPI_write_buffer[3];
      while(Flagbits.b_SPIDone==1);
      SPI_READ_Buffer[3]=SSPBUF0;
      ADC_MiddleByte=SPI_READ_Buffer[3];
      GPIO_PT2OutputHigh(PT24); //PT2.4 output high
      Flagbits.b_SPIDone= 0;
      Delay(200); //have to add, otherwise, write to HY17M26 SPI slave fail

      GPIO_PT2OutputLow(PT24); //PT2.4 output low
      Delay(20);      //have to add, this delay time is wait for HY17M26 SPI slave to prepare the data that needs to be transmitted
      SSPBUF0 = SPI_write_buffer[4];
      while(Flagbits.b_SPIDone==1);
      SPI_READ_Buffer[4]=SSPBUF0;
      ADC_LowByte=SPI_READ_Buffer[4];
      GPIO_PT2OutputHigh(PT24); //PT2.4 output high
      Flagbits.b_SPIDone= 0;
      Delay(200); //have to add, otherwise, write to HY17M26 SPI slave fail

      ADCData=(ADC_HighByte<<24)+(ADC_MiddleByte<<16)+(ADC_LowByte<<8);  //first, get temp 32bit ADC
      //ADCData=ADCData>>8;  //second, get 24bit ADC
    }

  }

}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
  NOP();
  //SPI Event
  if(SPIIF_IsFlag())
  {
    SPI_Test_counter++;

    SPIIF_ClearFlag();   //Clear SPIIF
    Flagbits.b_SPIDone= 1;
  }

}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void GPIOInit(void)
{
  //PT2.0 Input mode
  GPIO_PT2InputMode(PT20);
  GPIO_PT2DigitalEnable(PT20);

}

/*----------------------------------------------------------------------------*/
/* SPI Init Function                                                                */
/*----------------------------------------------------------------------------*/
void SPIInit(void)
{

#if defined(SPI_PT11)
  GPIO_GSPISet(GSPI_PT11);
  GPIO_PT1OutputMode(PT10); //PT1.0 output mode
  GPIO_PT1DigitalEnable(PT10);
  GPIO_PT1OutputHigh(PT10); //PT1.0 output high
#endif
#if defined(SPI_PT25)
  GPIO_GSPISet(GSPI_PT25);
  GPIO_PT2OutputMode(PT24); //PT2.4 output mode
  GPIO_PT2DigitalEnable(PT24);
  GPIO_PT2OutputHigh(PT24); //PT2.4 output high
#endif
#if defined(SPI_PT31)
  GPIO_GSPISet(GSPI_PT31);
  GPIO_PT3OutputMode(PT30); //PT3.0 output mode
  GPIO_PT3DigitalEnable(PT30);
  GPIO_PT3OutputHigh(PT30); //PT3.0 output high
#endif

  SPI_Open(SSPM_MCPUCK,CKP_HI,CKE_IDLE);  //master, 3-wire, SPI_CK=CPU_CK
                                          //CKP=1b
                                          //CKE=1b


}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
