#define  USE_HY17M26_2M
/******************************************************************************
 * SPI_Slave_Demo.c                                                           *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *       HY17M26(slave)            host                                       *
 *-------------------         ------------------                              *
 *                   |       |                                                *
 *                   |       |                                                *
 *               VDD | <-----| VDD                                            *
 *             PT2.4 | <-----| CS                                             *
 *             PT2.5 | <-----| SCL                                            *
 *             PT2.6 | ----->| MISO                                           *
 *             PT2.7 | <-----| MOSI                                           *
 *               VSS | <-----| VSS                                            *
 *                   |       |                                                *
 *                   |       |                                                *
 *-------------------         -----------------                               *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : SPI_Slave                                                   *
 * Created Date : 2021/6/4 By Cyril                                           *
 *                                                                            *
 ******************************************************************************/


/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <GPIO.h>
#include <CLK.h>
#include <ADC.h>
#include <PWR.h>
#include <SPI.h>
#include <RST.h>
#include <INT.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/

#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_17MHZ
#endif

//SPI PORT
//#define SPI_PT11    //PT10 CS, PT11 CK, PT12 MISO, PT13 MOSI
#define SPI_PT25  //PT24 CS, PT25 CK, PT26 MISO, PT27 MOSI
//#define SPI_PT31  //PT30 CS, PT31 CK, PT32 MISO, PT33 MOSI
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void CLOCKInit(void);
void ADCInit(void);
void SPI_SLAVE_Init(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
volatile typedef union _Flag
{
 unsigned int  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_SPIdone:1;
    unsigned Reserved:14;
  };
} Flag;
Flag   Flagbits;
//signed long
//range 8388607~-8388608 (dec)
//range 0x007F FFFF ~ 0xFF80 0000 (hex)
signed long  ADC_ISRTemp;
unsigned char ADC_LowByte;
unsigned char ADC_MiddleByte;
unsigned char ADC_HighByte;
unsigned int SPI_read_buffer[8];
unsigned int SPI_write_buffer[8];
unsigned char SPI_read_counter;
unsigned char SPI_write_counter;
unsigned char SPI_adc_send,SPI_Fail_Flag;

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
  // : SPI HosteX0xF0, 0x1F. HY17M26 SPI Slave^0x12, 0x34åBoɭԳ]wSPI_adc_send=1.
  //SPI_adc_send=1, SPI HostHNeX24bits(8bit+8bit+8bit), oɭSPI Slave|̧Ǧ^SPI_write_buffer[2]~[4]

  CLOCKInit();
  ADCInit();
  SPI_SLAVE_Init();

  SPI_write_buffer[0]=0x12;
  SPI_write_buffer[1]=0x34;
  SPI_write_buffer[2]=0xFF;
  SPI_write_buffer[3]=0xFF;
  SPI_write_buffer[4]=0xFF;
  SPI_adc_send=0;
  SPI_write_counter=0;
  SPI_read_counter=0;
  ADC_LowByte=0;
  ADC_MiddleByte=0;
  ADC_HighByte=0;
  Flagbits.b_ADCdone=0;
  Flagbits.b_SPIdone=0;

  GIE_Enable();

  while(1)
  {

    if(Flagbits.b_ADCdone== 1)
    {
      Flagbits.b_ADCdone= 0;
      ADC_HighByte=(ADC_ISRTemp >> 16) & 0xFF;
      ADC_MiddleByte=(ADC_ISRTemp >> 8) & 0xFF;
      ADC_LowByte=(ADC_ISRTemp >> 0) & 0xFF;
      SPI_write_buffer[2]=ADC_HighByte;
      SPI_write_buffer[3]=ADC_MiddleByte;
      SPI_write_buffer[4]=ADC_LowByte;
    }

    if(GPIO_PT2GET(PT24)==0)  //PT24=CSLow
    {
      ADC_Disable();
      //SPI slave write
      SSPBUF0=SPI_write_buffer[SPI_write_counter];  //SPI Write, ~|iJSPI_
      while(Flagbits.b_SPIdone== 0);
      SPI_write_counter++;
      Flagbits.b_SPIdone= 0;
      if(SPI_adc_send== 1)        //Seend ADC Data
      {
        if(SPI_write_counter>= 5)
        {
          SPI_write_counter= 2;
          SPI_read_counter= 2;
        }
      }
      else                       //Send 0x12 0x34
      {
        if(SPI_write_counter>= 2)
        {
          SPI_write_counter= 0;
        }
        if(SPI_read_counter>= 2)
        {
          SPI_read_counter= 0;
          if(SPI_read_buffer[0]==0xF0 && SPI_read_buffer[1]==0x1F)
          {
            SPI_adc_send =1;       //start to send ADC data
            SPI_write_counter= 2;  //]wwrite counterq2}l
            SPI_read_counter= 2;
          }
        }
      }
      while(GPIO_PT2GET(PT24)==0);  //wait SPI end
      ADC_Enable();
    }
  }



}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
  NOP();
  //ADC Event
  if(ADCIF_IsFlag())  //if ADCIF=1b
  {
    ADCIF_ClearFlag();   //set ADCIF=0b
    ADC_ISRTemp=ADCR; //ADC_GetData();
    Flagbits.b_ADCdone=1;
  }

  //SPI Event
  if(SPIIF_IsFlag())  //if SPIIF=1b
  {
    SPIIF_ClearFlag();   //Clear SPIIF
    SPI_read_buffer[SPI_read_counter++]=SSPBUF0;
    if(SPI0_BFCheck()==1)
    {
      SPI_Fail_Flag=1;

    }
    if(SPI0_POVCheck()==1)
    {
      SPI_Fail_Flag=1;
      SPI0_ClearPOV();
    }
    Flagbits.b_SPIdone= 1;
  }

}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}
/*----------------------------------------------------------------------------*/
/* SPI SLAVE Init Function                                                    */
/*----------------------------------------------------------------------------*/
void SPI_SLAVE_Init(void)
{
  //GPIO Init
#if defined(SPI_PT11)
  GPIO_GSPISet(GSPI_PT11);
  GPIO_PT1InputMode(PT10); //PT1.0 output mode
#endif
#if defined(SPI_PT25)
  GPIO_GSPISet(GSPI_PT25);
  GPIO_PT2InputMode(PT24); //PT2.4 input mode
  GPIO_PT2DigitalEnable(PT24);
  GPIO_PT2SETPU(PT24);
#endif
#if defined(SPI_PT31)
  GPIO_GSPISet(GSPI_PT31);
  GPIO_PT3InputMode(PT30); //PT3.0 output mode
#endif

  SPI_Open(SSPM_S3WIRE,CKP_HI,CKE_IDLE);  //master, 3-wire, SPI_CK=CPU_CK
                                          //CKP=1b
                                          //CKE=1b

}

/*----------------------------------------------------------------------------*/
/* ADC Init Function                                                          */
/*----------------------------------------------------------------------------*/
void ADCInit(void)
{
   //==========Setup Power=====================================
  PWR_BGREnable();			//bandgap Vref Enable
  PWR_VDDAOpen(LDOC_2V4);	//VDDA Enable, refer to the Datasheet for LDOC typical voltage

  //==========Setup ADC Clock=================================
#if defined(HAO_2MHZ)
  //HAO=2MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV2,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_4MHZ)
  //HAO=4MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV4,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_8MHZ)
  //HAO=8MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV8,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_16MHZ)
  //HAO=17MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV16,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif


#if defined ADC_ChopperMode
  CSFON_Enable();
  ADC_ENINXCH_Enable();
  ADC_DAFM_CHOPPER();
  ADC_ENCH_Enable();
  ADC_Enable();
  ADC_CMFREnable();     //CMFR=1, Comb Filter Reset
#endif

}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
