#define  USE_HY17M26_2M
/******************************************************************************
 * TMA2_Demo.c                                                                *
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        HY17M26                                                             *
 *--------------------                                                        *
 *                    |                                                       *
 *               PT2.0|->_|-|_ TMA2                                           *
 *                    |                                                       *
 *               PT2.1|->_|-|_ TMA2C                                          *
 *                    |                                                       *
 *--------------------                                                        *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M26                                                           *
 * Project Name : TMA2                                                        *
 * Created Date : 2021/6/4 BY Cyril                                           *
 *                                                                            *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <CLK.h>
#include <GPIO.h>
#include <RST.h>
#include <TMR.h>

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void TMAInit(void);
void CLOCKInit(void);
void GPIOInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned int TMA2R_Count;
unsigned int TMA2C_Count;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
    CLOCKInit();
    GPIOInit();
    TMAInit();
    TMA2R_Count=0;
    TMA2C_Count=0;

    GIE_Enable();

    while(1);

}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
	if(TA2IF_IsFlag())       //TA1IF=1,TMA1R++
    {
        PT2= PT2^PT20_MSK;  //PT2.0 toggle
        TMA2R_Count++;
        TA2IF_ClearFlag();
    }

	if(TA2CIF_IsFlag())        //TMA1C=TMA1R ,then TA1CIF=1
    {
        PT2= PT2^PT21_MSK;  //PT2.1 toggle
        TMA2C_Count++;
        TMA2_ClearTMA2R();  //clear TMA1R
        TA2CIF_ClearFlag();
	}
}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
  CLK_DMSCKSelect(DMS_DHSCKDIV2);  //DMS_CK= HAO/1/2
}

/*----------------------------------------------------------------------------*/
/* TMA Init Function                                                          */
/*----------------------------------------------------------------------------*/
void TMAInit(void)
{
  TMA2_Open(TMAS_DMSCK,DTMA_TMACKDIV2,100); //DTMA_CK=TMA_CK/256/2
                                            //TMAxC_Flag=TMAx_Flag*100
  TA2CIF_ClearFlag();
  TA2CIE_Enable();
  TA2IF_ClearFlag();
  TA2IE_Enable();
}

/*----------------------------------------------------------------------------*/
/* GPIO Init Function                                                         */
/*----------------------------------------------------------------------------*/
void GPIOInit(void)
{
  GPIO_PT2OutputMode(PT20);
  GPIO_PT2DigitalEnable(PT20);
  GPIO_PT2OutputLow(PT20);

  GPIO_PT2OutputMode(PT21);
  GPIO_PT2DigitalEnable(PT21);
  GPIO_PT2OutputLow(PT21);
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
