#define  USE_HY17M26_2M
/*******************************************************************************
 * WDT_Demo.c                                                                  *
 * --------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                             *
 * http://www.hycontek.com/                                                    *
 *                                                                             *
 * Program Description:                                                        *
 *                                                                             *
 * Test_IDLE: IDLE mode. WDT count overflow, wake up from IDLE, WDTIF=1        *
 * Test_CLR : Normal mode. Clear WDT count, IC doesn't reset                   *
 * Test_NOP: Normal mode. Not clear WDT count,IC will reset, TO=1              *
 *                                                                             *
 * For External Input                                                          *
 * IC Body: HY17M26                                                            *
 * Project Name : WDT                                                          *
 * Created Date : 2021/6/4 BY Cyril                                            *
 *                                                                             *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <WDT.h>
#include <INT.h>
#include <RST.h>
#include <CLK.h>
#include <GPIO.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M26_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M26_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M26_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M26_16M
#define HAO_16MHZ
#endif

//#define Test_IDLE
//#define Test_CLR
#define Test_RESET

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void WDTInit(void);
void CLOCKInit(void);
void GPIOInit(void);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
    CLOCKInit();
    GPIOInit();
    WDTInit();
    GIE_Enable();


#ifdef Test_IDLE    //wake up Idle
    while(1)
    {
      WDT_WDTCKDivSelect(DWDT_WDTCKDIV64);
	  Idle();
	  NOP();
      PT2= PT2^PT20_MSK;  //PT2.0 toggle
     }
#endif

#ifdef Test_CLR   //clear WDT Count, won't reset
    while(1)
    {
      if(SYS_ReadWDT()!=0)  //if program reset, TO=1
      {
      	SYS_ClearWDT();
      	PT2= PT2^PT20_MSK;  //PT2.0 toggle
      }
      WDT_Clear();
    }
#endif

#ifdef Test_RESET   //nop, will reset
    while(1)
    {
      if(SYS_ReadWDT()!=0)  //if program reset, TO=1
      {
      	SYS_ClearWDT();
      	PT2= PT2^PT20_MSK;  //PT2.0 toggle
      }
      NOP();
    }
#endif

}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt  //WDT Reset
{
	WDTIF_ClearFlag();
	SYS_ClearIDLE();	//CLR IDLE Flag
	WDT_Clear();		//CLR WDT count.
}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* WDT Init Function                                                          */
/*----------------------------------------------------------------------------*/
void WDTInit(void)
{
  WDT_Open(DWDT_WDTCKDIV2048); 	 //WDT Open
  WDT_Clear();                     // CLEAR WDT COUNT
  WDTIF_ClearFlag();
  WDTIE_Enable();
  WDT_Enable();
}

/*----------------------------------------------------------------------------*/
/* GPIO Init Function                                                         */
/*----------------------------------------------------------------------------*/
void GPIOInit(void)
{
  GPIO_PT2OutputMode(PT20);
  GPIO_PT2DigitalEnable(PT20);
  GPIO_PT2OutputLow(PT20);

}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
